# -*- coding: utf-8 -*-

import os
from .base import BaseBackend
import shutil


class LocalBackend(BaseBackend):
    """
    本地后端
    """

    host = None
    port = None
    username = None
    password = None
    remote_dir = None

    def __init__(self, remote_dir):
        super(LocalBackend, self).__init__()
        self.remote_dir = remote_dir

    def upload(self, file_path):
        """
        上传
        """
        if not os.path.exists(self.remote_dir):
            os.makedirs(self.remote_dir)

        if os.path.isfile(self.remote_dir):
            raise ValueError('remote_dir should be dir. %s' % self.remote_dir)

        shutil.copy(file_path, self.remote_dir)



# -*- coding: utf-8 -*-

import os
import paramiko
from .base import BaseBackend


class SSHBackend(BaseBackend):
    """
    ssh后端
    """

    host = None
    port = None
    username = None
    password = None
    remote_dir = None

    def __init__(self, host, port, username, password, remote_dir):
        super(SSHBackend, self).__init__()
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.remote_dir = remote_dir

    def upload(self, file_path):
        """
        上传
        """
        filename = os.path.basename(file_path)

        transport = paramiko.Transport((self.host, self.port))
        transport.connect(username=self.username, password=self.password)
        sftp = paramiko.SFTPClient.from_transport(transport)

        remote_path = os.path.join(self.remote_dir, filename)

        sftp.put(file_path, remote_path)



# -*- coding: utf-8 -*-

import os
from baidupcs import PCS, tools

from .base import BaseBackend


class BaiDuYunBackend(BaseBackend):

    pcs = None
    refresh_token = None
    remote_dir = None
    client_id = None
    client_secret = None

    def __init__(self, access_token, remote_dir=None,
                 refresh_token=None,
                 client_id=None,
                 client_secret=None
        ):
        """
        access_token: 访问数据token。有效期1个月
        remote_dir: 对端要存放的目录地址
        refresh_token: refresh_token。有效期10年
        client_id: 更新access_token需要
        client_secret: 更新access_token需要
        """
        super(BaiDuYunBackend, self).__init__()

        self.pcs = PCS(access_token)
        self.remote_dir = remote_dir or '/apps/'
        self.refresh_token = refresh_token
        self.client_id = client_id
        self.client_secret = client_secret

    def refresh_access_token(self):
        """
        获取新的refresh token
        """
        rsp = tools.get_new_access_token(self.refresh_token, self.client_id, self.client_secret)

        access_token = rsp.json().get('access_token')
        self.pcs = PCS(access_token)

    def upload(self, file_path):
        """
        上传
        """
        filename = os.path.basename(file_path)
        remote_path = os.path.join(self.remote_dir, filename)

        local_file = open(file_path, 'rb')

        rsp = self.pcs.upload(remote_path, local_file)
        if rsp.status_code == 401 and rsp.json().get('error_code') in (110, 111):
            self.refresh_access_token()
            rsp = self.pcs.upload(remote_path, local_file)

        local_file.close()

        return rsp
